/*
 *  This file is part of the SNPack library.
 *  Copyright (c) Francesco Sambo <sambofra@dei.unipd.it>
 *
 *  SNPack is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SNPack is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
 *  the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _PACK_H_
#define _PACK_H_

/**
 * \file  pack.h
 * \brief Definition of the Pack class.
 */

#include <vector>
#include <algorithm>
#include "info.h"
#include "data.h"

#define BASEJIT 10

using std::vector;
using std::ofstream;

/// Utility class for compressing and saving SNP data to disk 
class Pack{
	 
 public:
	
	/// Computes gain for each SNP and best summary for compression 
	Pack( Info& infoRef, Data& dataRef, bool verbose = true );
 
	/* public, for possible future threading */
	/// Write compressed data to output file stream
	uint_fast32_t saveToPck( ofstream& out );
	
	/// Writes compression statistics to output file stream
	void writeStat( ofstream& stat );

 private:	
	
	void computeGain( uint_fast32_t ck );
	void computeSummary( uint_fast32_t ck );
	
	uint_fast32_t parMaxGain();
	
	Info& info;
	Data& data;
	bool verb;
	
	uint_fast32_t nSubj;
	uint_fast32_t nSnps;
	uint_fast32_t nChunk;
	
	/* from _info_ */
	vector<uint32_t>& sLN;
	vector<uint32_t>& sRN;
	vector<uint32_t>& ckStart;
	uint_fast8_t nBitSubj;
	
	/* Relative index of summarizing SNP, or 0 */
	vector<int32_t> summary;
		
	vector<vector<uint32_t> > uNeigh;
	vector<vector<uint32_t> > uNeighDiff;
	vector<uint32_t> gain;
	vector<uint32_t> byteCost;
};

#endif
