/*
 *  This file is part of the SNPack library.
 *  Copyright (c) Francesco Sambo <sambofra@dei.unipd.it>
 *
 *  SNPack is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SNPack is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
 *  the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * \file  util.h
 * \brief Utility functions and exceptions.
 */

#ifndef _UTIL_H_
#define _UTIL_H_

#include <vector>
#include <cstdio>
#include <string>
#include <stdint.h>
#include <stdexcept>
#include <iterator>
#include <fstream>

/// Bad index exception
class BadIndex: public std::out_of_range
{
	public:
	   BadIndex(const char * error) : std::out_of_range(error) { }
};

/// Bad value exception
class BadValue: public std::out_of_range
{
	public:
	   BadValue(const char * error) : std::out_of_range(error) { }
};

/// Bad file format exception
class BadFileFormat: public std::runtime_error
{
	public:
	   BadFileFormat(const char * error) : std::runtime_error(error) { }
};

/// Bad allocation exception
class BadAlloc: public std::runtime_error
{
	public:
	   BadAlloc(const char * error) : std::runtime_error(error) { }
};

/// Compare two files on disk
bool compare_files(std::string fname1, std::string fname2);

#endif
